#ifndef _IPCONTACT_APP_H
#define _IPCONTACT_APP_H

#include <asterisk.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/version.h>

#include <mysql/mysql.h>

#define IPCONTACT_ADDONS_VERSION "1.4.4"
#define IPCONTACT_CONFDIR "/etc/asterisk"

#define	IPC_EXTEN_NUMBER "number"
#define	IPC_EXTEN_INFO "info"
#define	IPC_EXTEN_PROTO "protocol"
#define IPC_EXTEN_CHANNEL "channel"
#define IPC_EXTEN_VM "voicemail"
#define	IPC_EXTEN_VMFLAG "vm_flag"
#define	IPC_EXTEN_MONITOR "monitor"

#define IPC_PROFILE_LOCAL "local"
#define IPC_PROFILE_CELLULAR "cellular"
#define IPC_PROFILE_LONGDIST "longdist"
#define IPC_PROFILE_INTER "inter"

/* Maximum variable lengths */
// AST_MAX_APP 32 - defined in pbx.h
// AST_MAX_EXTENSION 80 - defined in channel.h
// AST_MAX_CONTEXT 80 - defined in channel.h
#define IPC_MAX_LEN 128
#define IPC_QUERY_LEN 256
#define IPC_MYSQL_PING_TIME 10

/* Macros */
#define CHAN_REAL_EXTEN(ch)  (ast_strlen_zero(ch->macroexten)?ch->exten:ch->macroexten)
#define IS_INCOMING_CALL(ch) ast_true(pbx_builtin_getvar_helper(chan, "INCOMING_CALL"))
#define IS_INCOMINGCELL_CALL(ch) ast_true(pbx_builtin_getvar_helper(ch, "INCOMING_CELL"))

/* Version conditionals */
#ifdef ASTERISK_VERSION_NUM

#if (ASTERISK_VERSION_NUM >= 10600)
#define HAS_AST16
#warning "Using Asterisk 1.6 API"
#else 
#if (ASTERISK_VERSION_NUM >= 10400)
#warning "Using Asterisk 1.4 API"
#else
#error "Asterisk 1.2 is not supported!"
#endif
#endif

#else
#error "ASTERISK_VERSION_NUM is not defined. Please install asterisk-devel package"
#endif

/* Asterisk argument delimiter, will be defined someday (see https://reviewboard.asterisk.org/r/79/) */
#ifndef AST_STANDARD_APP_DELIM
#ifdef HAS_AST16
#define AST_STANDARD_APP_DELIM ","
#else
#define AST_STANDARD_APP_DELIM "|"
#endif
#endif

const char *IPC_EXTEN_TREE = "ipcontact/extension";

struct ast_ipcontact_conf {
	char dbuser[AST_MAX_APP];
	char dbpass[AST_MAX_APP];
	char dbhost[AST_MAX_APP];
	char dbname[AST_MAX_APP];
	char tables[IPC_MAX_LEN];
};

struct ast_ipcontact_table
{
	char table[AST_MAX_APP];
	AST_LIST_ENTRY(ast_ipcontact_table) list; /*   Next table in list */
};


struct virtual_pbx {
	int number;
	int family;
	char phone[AST_MAX_EXTENSION];
	char name[AST_MAX_EXTENSION];
	char mainmenu[AST_MAX_CONTEXT];
	char operator[AST_MAX_CONTEXT];
	char vmslocal[AST_MAX_CONTEXT];
	char vmsincoming[AST_MAX_CONTEXT];
	char dialoptsloc[AST_MAX_APP];
	char dialoptsext[AST_MAX_APP];
	char dialoptsret[AST_MAX_APP];
	int monitor;
	/*   Next virtual pbx in list */
	AST_LIST_ENTRY(virtual_pbx) list;
};

/**** MySQL functions ****/
MYSQL *ipc_sql_reconnect(void);
void ipc_sql_disconnect(void);

/* ipc_sql_exec_query(char* query):
   Recibe un string con la consulta a realizar en la base de datos
   y devuelve un puntero a MYSQL_RES con el resultado o NULL si se
   produjo un error.
*/
MYSQL_RES *ipc_sql_exec_query(char *query);

int ipc_mutex_lock(void);
int ipc_mutex_unlock(void);

struct virtual_pbx *get_virtual_pbx(int vpbx);
int get_exten_profile(const char *exten, const char *profile);

int is_number(const char *num);

const char *ast_channel_type(struct ast_channel *chan);

#endif
